@@ -86,6 +86,7 @@
 #include "lq_plugin.h"
 #include "common/autobuf.h"
 #include "gateway.h"
+#include "egressTypes.h"
 
 #include "olsrd_jsoninfo.h"
 #include "olsrd_plugin.h"
@@ -96,6 +97,9 @@
 
 static int ipc_socket;
 
+/* Response types */
+#define HTTP_200 "HTTP/1.1 200 OK"
+
 /* IPC initialization function */
 static int plugin_ipc_init(void);
 
@@ -126,6 +130,18 @@ static void ipc_print_interfaces(struct autobuf *);
 static void ipc_print_plugins(struct autobuf *);
 static void ipc_print_olsrd_conf(struct autobuf *abuf);
 
+static size_t build_http_header(const char *status, const char *mime,
+  uint32_t msgsize, char *buf, uint32_t bufsize);
+
+/*
+ * this is the size of the buffer used for build_http_header
+ * the amount of data written into the buffer will be less than
+ * 400 bytes approximatively.
+ * The size may vary because the Content-Length header contains
+ * the length of the json data
+ */
+#define MAX_HTTPHEADER_SIZE 512
+
 #define TXT_IPC_BUFSIZE 256
 
 /* these provide all of the runtime status info */
@@ -370,6 +386,14 @@ plugin_ipc_init(void)
       return 0;
     }
 #endif /* (defined __FreeBSD__ || defined __FreeBSD_kernel__) && defined SO_NOSIGPIPE */
+#if defined linux && defined IPV6_V6ONLY
+    if (jsoninfo_ipv6_only && olsr_cnf->ip_version == AF_INET6) {
+      if (setsockopt(ipc_socket, IPPROTO_IPV6, IPV6_V6ONLY, (char *)&yes, sizeof(yes)) < 0) {
+        perror("IPV6_V6ONLY failed");
+        return 0;
+      }
+    }
+#endif /* defined linux && defined IPV6_V6ONLY */
     /* Bind the socket */
 
     /* complete the socket structure */
@@ -508,8 +532,15 @@ ipc_action(int fd, void *data __attribute__ ((unused)), unsigned int flags __att
   FD_ZERO(&rfds);
   FD_SET((unsigned int)ipc_connection, &rfds);  /* Win32 needs the cast here */
   if (0 <= select(ipc_connection + 1, &rfds, NULL, NULL, &tv)) {
-    char requ[128];
-    ssize_t s = recv(ipc_connection, (void *)&requ, sizeof(requ), 0);   /* Win32 needs the cast here */
+    char requ[1024];
+    ssize_t s = recv(ipc_connection, (void *)&requ, sizeof(requ)-1, 0);   /* Win32 needs the cast here */
+
+    if (s == sizeof(requ)-1) {
+      /* input was too much long, just skip the rest */
+      char dummy[1024];
+
+      while (recv(ipc_connection, (void *)&dummy, sizeof(dummy), 0) == sizeof(dummy));
+    }
     if (0 < s) {
       requ[s] = 0;
       /* print out the requested tables */
@@ -1006,8 +1037,8 @@ ipc_print_config(struct autobuf *abuf)
       abuf_json_string(abuf, "smartGatewayEgressInterfaces", egressbuf.buf);
       abuf_free(&egressbuf);
     }
-    abuf_json_int(abuf, "smartGatewayMarkOffsetEgress", olsr_cnf->smart_gw_mark_offset_egress);
-    abuf_json_int(abuf, "smartGatewayMarkOffsetTunnels", olsr_cnf->smart_gw_mark_offset_tunnels);
+    abuf_json_int(abuf, "smartGatewayTablesOffset", olsr_cnf->smart_gw_offset_tables);
+    abuf_json_int(abuf, "smartGatewayRulesOffset", olsr_cnf->smart_gw_offset_rules);
     abuf_json_boolean(abuf, "smartGatewayAllowNat", olsr_cnf->smart_gw_allow_nat);
     abuf_json_boolean(abuf, "smartGatewayUplinkNat", olsr_cnf->smart_gw_uplink_nat);
     abuf_json_int(abuf, "smartGatewayPeriod", olsr_cnf->smart_gw_period);
@@ -1095,7 +1126,7 @@ ipc_print_interfaces(struct autobuf *abuf)
   const struct olsr_if *ifs;
   abuf_json_open_array(abuf, "interfaces");
   for (ifs = olsr_cnf->interfaces; ifs != NULL; ifs = ifs->next) {
-    const struct interface *const rifs = ifs->interf;
+    const struct interface_olsr *const rifs = ifs->interf;
     abuf_json_open_array_entry(abuf);
     abuf_json_string(abuf, "name", ifs->name);
 
@@ -1275,6 +1306,9 @@ static void
 send_info(unsigned int send_what, int the_socket)
 {
   struct autobuf abuf;
+  size_t header_len = 0;
+  char header_buf[MAX_HTTPHEADER_SIZE];
+  const char *content_type = "application/json";
 
   /* global variables for tracking when to put a comma in for JSON */
   entrynumber[0] = 0;
@@ -1303,9 +1337,9 @@ send_info(unsigned int send_what, int the_socket)
   if (send_what & SIW_ALL) {
     abuf_json_int(&abuf, "systemTime", time(NULL));
     abuf_json_int(&abuf, "timeSinceStartup", now_times);
-    if(*uuid != 0)
+    if (*uuid != 0)
       abuf_json_string(&abuf, "uuid", uuid);
-      abuf_puts(&abuf, "}\n");
+    abuf_puts(&abuf, "}\n");
   }
 
   /* this outputs the olsrd.conf text directly, not JSON */
@@ -1313,12 +1347,17 @@ send_info(unsigned int send_what, int the_socket)
     ipc_print_olsrd_conf(&abuf);
   }
 
-  outbuffer[outbuffer_count] = olsr_malloc(abuf.len, "txt output buffer");
-  outbuffer_size[outbuffer_count] = abuf.len;
+  if(http_headers) {
+    header_len = build_http_header(HTTP_200, content_type, abuf.len, header_buf, sizeof(header_buf));
+  }
+
+  outbuffer[outbuffer_count] = olsr_malloc(header_len + abuf.len, "json output buffer");
+  outbuffer_size[outbuffer_count] = header_len + abuf.len;
   outbuffer_written[outbuffer_count] = 0;
   outbuffer_socket[outbuffer_count] = the_socket;
 
-  memcpy(outbuffer[outbuffer_count], abuf.buf, abuf.len);
+  memcpy(outbuffer[outbuffer_count], header_buf, header_len);
+  memcpy((outbuffer[outbuffer_count]) + header_len, abuf.buf, abuf.len);
   outbuffer_count++;
 
   if (outbuffer_count == 1) {
@@ -1333,6 +1372,53 @@ send_info(unsigned int send_what, int the_socket)
   abuf_free(&abuf);
 }
 
+static size_t
+build_http_header(const char *status, const char *mime, uint32_t msgsize,
+  char *buf, uint32_t bufsize)
+{
+  time_t currtime;
+  size_t size;
+
+  size = snprintf(buf, bufsize, "%s\r\n", status);
+
+  /* Date */
+  time(&currtime);
+  size += strftime(&buf[size], bufsize - size, "Date: %a, %d %b %Y %H:%M:%S GMT\r\n", localtime(&currtime));
+
+  /* Server version */
+  size += snprintf(&buf[size], bufsize - size, "Server: OLSRD JSONInfo plugin\r\n");
+
+  /* connection-type */
+  size += snprintf(&buf[size], bufsize - size, "Connection: closed\r\n");
+
+  /* MIME type */
+  if(mime != NULL) {
+    size += snprintf(&buf[size], bufsize - size, "Content-type: %s\r\n", mime);
+  }
+
+  /* CORS data */
+  /**No needs to be strict here, access control is based on source IP*/
+  size += snprintf(&buf[size], bufsize - size, "Access-Control-Allow-Origin: *\r\n");
+  size += snprintf(&buf[size], bufsize - size, "Access-Control-Allow-Methods: GET, POST, OPTIONS\r\n");
+  size += snprintf(&buf[size], bufsize - size, "Access-Control-Allow-Headers: Accept, Origin, X-Requested-With\r\n");
+  size += snprintf(&buf[size], bufsize - size, "Access-Control-Max-Age: 1728000\r\n");
+
+  /* Content length */
+  if (msgsize > 0) {
+    size += snprintf(&buf[size], bufsize - size, "Content-length: %i\r\n", msgsize);
+  }
+
+  /* Cache-control
+   * No caching dynamic pages
+   */
+  size += snprintf(&buf[size], bufsize - size, "Cache-Control: no-cache\r\n");
+
+  /* End header */
+  size += snprintf(&buf[size], bufsize - size, "\r\n");
+
+  return size;
+}
+
 /*
  * Local Variables:
  * mode: c
