@@ -64,6 +64,8 @@ union olsr_ip_addr jsoninfo_accept_ip;
 union olsr_ip_addr jsoninfo_listen_ip;
 int ipc_port;
 int nompr;
+bool http_headers;
+int jsoninfo_ipv6_only;
 
 static void my_init(void) __attribute__ ((constructor));
 static void my_fini(void) __attribute__ ((destructor));
@@ -79,6 +81,9 @@ my_init(void)
 
   /* defaults for parameters */
   ipc_port = 9090;
+  http_headers = false;
+  jsoninfo_ipv6_only = false;
+
   if (olsr_cnf->ip_version == AF_INET) {
     jsoninfo_accept_ip.v4.s_addr = htonl(INADDR_LOOPBACK);
     jsoninfo_listen_ip.v4.s_addr = htonl(INADDR_ANY);
@@ -120,11 +125,27 @@ store_string(const char *value, void *data, set_plugin_parameter_addon addon __a
   return 0;
 }
 
+static int
+store_boolean(const char *value, void *data, set_plugin_parameter_addon addon __attribute__ ((unused)))
+{
+  bool *dest = data;
+  if(strcmp(value, "yes") == 0)
+    *dest = true;
+  else if (strcmp(value, "no") == 0)
+    *dest = false;
+  else
+    return 1; //error
+
+  return 0;
+}
+
 static const struct olsrd_plugin_parameters plugin_parameters[] = {
   {.name = "port",.set_plugin_parameter = &set_plugin_port,.data = &ipc_port},
   {.name = "accept",.set_plugin_parameter = &set_plugin_ipaddress,.data = &jsoninfo_accept_ip},
   {.name = "listen",.set_plugin_parameter = &set_plugin_ipaddress,.data = &jsoninfo_listen_ip},
   {.name = "uuidfile",.set_plugin_parameter = &store_string,.data = uuidfile},
+  {.name = "httpheaders",.set_plugin_parameter = &store_boolean,.data = &http_headers},
+  {.name = "ipv6only", .set_plugin_parameter = &set_plugin_boolean, .data = &jsoninfo_ipv6_only},
 };
 
 void
