@@ -52,7 +52,6 @@
 #include <fcntl.h>
 #include <linux/if_ether.h>     /* ETH_P_IP */
 #include <linux/if_packet.h>    /* struct sockaddr_ll, PACKET_MULTICAST */
-//#include <pthread.h> /* pthread_t, pthread_create() */
 #include <signal.h>             /* sigset_t, sigfillset(), sigdelset(), SIGINT */
 #include <netinet/ip.h>         /* struct ip */
 #include <netinet/udp.h>        /* struct udphdr */
@@ -86,6 +85,7 @@
 
 int P2pdTtl                        = 0;
 int P2pdUseHash                    = 0;  /* Switch off hash filter by default */
+int P2pdUseTtlDecrement            = 0;  /* No TTL decrement by default */
 int P2pdDuplicateTimeout           = P2PD_VALID_TIME;
 
 /* List of UDP destination address and port information */
@@ -102,7 +102,7 @@ char * get_ipv4_str(uint32_t address, char *s, size_t maxlen);
 char * get_ipv6_str(unsigned char* address, char *s, size_t maxlen);
 #ifdef INCLUDE_DEBUG_OUTPUT
 void dump_packet(unsigned char* packet, int length);
-#endif
+#endif /* INCLUDE_DEBUG_OUTPUT */
 bool check_and_mark_recent_packet(unsigned char *data, int len);
 
 /* -------------------------------------------------------------------------
@@ -217,7 +217,7 @@ PacketReceivedFromOLSR(unsigned char *encapsulationUdpData, int len)
                                      sizeof(tmp)),
                         destPort);
           }
-#endif
+#endif /* INCLUDE_DEBUG_OUTPUT */
         }
       } else /* (olsr_cnf->ip_version == AF_INET6) */ {
         if (ip6Header->ip6_nxt == SOL_UDP && !IsIpv6Fragment(ip6Header)) {
@@ -236,7 +236,7 @@ PacketReceivedFromOLSR(unsigned char *encapsulationUdpData, int len)
                                      sizeof(tmp)),
                         destPort);
           }
-#endif
+#endif /* INCLUDE_DEBUG_OUTPUT */
         }
       }
 
@@ -409,7 +409,7 @@ p2pd_is_duplicate_message(union olsr_message *msg)
  * ------------------------------------------------------------------------- */
 bool
 olsr_parser(union olsr_message *m,
-            struct interface *in_if __attribute__ ((unused)),
+            struct interface_olsr *in_if __attribute__ ((unused)),
             union olsr_ip_addr *ipaddr __attribute__ ((unused)))
 {
   union olsr_ip_addr originator;
@@ -460,7 +460,7 @@ olsr_p2pd_gen(unsigned char *packet, int len)
   char buffer[10240];
   int aligned_size;
   union olsr_message *message = (union olsr_message *)buffer;
-  struct interface *ifn;
+  struct interface_olsr *ifn;
 
   aligned_size=len;
 
@@ -528,7 +528,7 @@ P2pdPError(const char *format, ...)
 
 #if !defined REMOVE_LOG_DEBUG
   char *stringErr = strerror(errno);
-#endif
+#endif /* !defined REMOVE_LOG_DEBUG */
 
   /* Rely on short-circuit boolean evaluation */
   if (format == NULL || *format == '\0') {
@@ -544,7 +544,7 @@ P2pdPError(const char *format, ...)
 
 #if !defined REMOVE_LOG_DEBUG
     OLSR_DEBUG(LOG_PLUGINS, "%s: %s\n", strDesc, stringErr);
-#endif
+#endif /* !defined REMOVE_LOG_DEBUG */
   }
 }                               /* P2pdPError */
 
@@ -604,6 +604,46 @@ InUdpDestPortList(int ip_version, union olsr_ip_addr *addr, uint16_t port)
   return false;
 }
 
+/*
+ * Function for checksum calculation.
+ * From the RFC, the checksum algorithm is:
+ *   "The checksum field is the 16 bit one's complement of the one's
+ *   complement sum of all 16 bit words in the header. For purposes of
+ *   computing the checksum, the value of the checksum field is zero."
+ *
+ * For example, consider Hex 4500003044224000800600008c7c19acae241e2b (20 bytes IP header):
+ * - Step 1) 4500 + 0030 + 4422 + 4000 + 8006 + 0000 + 8c7c + 19ac + ae24 + 1e2b = 0002`BBCF (16-bit sum)
+ * - Step 2) 0002 + BBCF = BBD1 = 1011101111010001 (1's complement 16-bit sum)
+ * - Step 3) ~BBD1 = 0100010000101110 = 442E (1's complement of 1's complement 16-bit sum)
+ */
+static void recomputeIPv4HeaderChecksum(struct ip *header) {
+  uint32_t sum;
+  uint32_t nwords;
+  u_short *headerWords;
+
+  if (!header) {
+    return;
+  }
+
+  header->ip_sum = 0;
+  nwords = header->ip_hl << 1;
+  headerWords = (u_short *) header;
+
+  /* step 1 */
+  for (sum = 0; nwords > 0; nwords--) {
+    sum += ntohs(*headerWords);
+    headerWords++;
+  }
+
+  /* step 2 */
+  sum = (sum >> 16) + (sum & 0xffff);
+
+  /* step 3 */
+  sum = ~sum & 0xffff;
+
+  header->ip_sum = htons((u_short)sum);
+}
+
 /* -------------------------------------------------------------------------
  * Function   : P2pdPacketCaptured
  * Description: Handle a captured IP packet
@@ -620,9 +660,11 @@ static void
 P2pdPacketCaptured(unsigned char *encapsulationUdpData, int nBytes)
 {
   union olsr_ip_addr dst;      /* Destination IP address in captured packet */
-  struct ip *ipHeader;         /* The IP header inside the captured IP packet */
+  struct ip *ipHeader = NULL;  /* The IP header inside the captured IP packet */
   struct ip6_hdr *ipHeader6;   /* The IP header inside the captured IP packet */
   struct udphdr *udpHeader;
+  uint8_t * ttl = NULL;
+  int recomputeChecksum = 0;
   u_int16_t destPort;
 
   if ((encapsulationUdpData[0] & 0xf0) == 0x40) {       //IPV4
@@ -635,7 +677,7 @@ P2pdPacketCaptured(unsigned char *encapsulationUdpData, int nBytes)
       /* Not UDP */
 #ifdef INCLUDE_DEBUG_OUTPUT
       OLSR_PRINTF(1,"%s: NON UDP PACKET\n", PLUGIN_NAME_SHORT);
-#endif
+#endif /* INCLUDE_DEBUG_OUTPUT */
       return;                   /* for */
     }
 
@@ -644,7 +686,7 @@ P2pdPacketCaptured(unsigned char *encapsulationUdpData, int nBytes)
     if (IsIpv4Fragment(ipHeader)) {
 #ifdef INCLUDE_DEBUG_OUTPUT
       OLSR_PRINTF(1, "%s: Is IPv4 fragment\n", PLUGIN_NAME_SHORT);
-#endif
+#endif /* INCLUDE_DEBUG_OUTPUT */
       return;
     }
 
@@ -660,9 +702,12 @@ P2pdPacketCaptured(unsigned char *encapsulationUdpData, int nBytes)
       char tmp[32];
       OLSR_PRINTF(1, "%s: Not in dest/port list: %s:%d\n", PLUGIN_NAME_SHORT,
                   get_ipv4_str(dst.v4.s_addr, tmp, sizeof(tmp)), destPort);
-#endif
+#endif /* INCLUDE_DEBUG_OUTPUT */
        return;
     }
+
+    ttl = &ipHeader->ip_ttl;
+    recomputeChecksum = 1;
   }                            //END IPV4
   else if ((encapsulationUdpData[0] & 0xf0) == 0x60) {  //IPv6
 
@@ -686,7 +731,7 @@ P2pdPacketCaptured(unsigned char *encapsulationUdpData, int nBytes)
     if (IsIpv6Fragment(ipHeader6)) {
 #ifdef INCLUDE_DEBUG_OUTPUT
       OLSR_PRINTF(1, "%s: Is IPv6 fragment\n", PLUGIN_NAME_SHORT);
-#endif
+#endif /* INCLUDE_DEBUG_OUTPUT */
       return;
     }
 
@@ -701,14 +746,32 @@ P2pdPacketCaptured(unsigned char *encapsulationUdpData, int nBytes)
       char tmp[64];
       OLSR_PRINTF(1, "%s: Not in dest/port list: %s:%d\n", PLUGIN_NAME_SHORT,
                   get_ipv6_str(dst.v6.s6_addr, tmp, sizeof(tmp)), destPort);
-#endif
+#endif /* INCLUDE_DEBUG_OUTPUT */
       return;
     }
+
+    ttl = &ipHeader6->ip6_ctlun.ip6_un1.ip6_un1_hlim;
+    recomputeChecksum = 0;
   }                             //END IPV6
   else {
     return;                     //Is not IP packet
   }
 
+  if (P2pdUseTtlDecrement) {
+    assert(ttl);
+    if (!*ttl) {
+      return;
+    }
+    *ttl -= 1;
+    if (!*ttl) {
+      return;
+    }
+
+    if (recomputeChecksum) {
+      recomputeIPv4HeaderChecksum(ipHeader);
+    }
+  }
+
   // send the packet to OLSR forward mechanism
   olsr_p2pd_gen(encapsulationUdpData, nBytes);
 }                               /* P2pdPacketCaptured */
@@ -742,7 +805,7 @@ DoP2pd(int skfd,
                       0, (struct sockaddr *)&pktAddr, &addrLen);
 #ifdef INCLUDE_DEBUG_OUTPUT
     OLSR_PRINTF(1, "%s: Received %d bytes\n", PLUGIN_NAME_SHORT, nBytes);
-#endif
+#endif /* INCLUDE_DEBUG_OUTPUT */
 
     if (nBytes < 0) {
 
@@ -774,7 +837,7 @@ DoP2pd(int skfd,
       OLSR_PRINTF(1, "%s: Multicast or broadcast packet was captured.\n",
                   PLUGIN_NAME_SHORT);
       dump_packet(ipPacket, nBytes);
-#endif
+#endif /* INCLUDE_DEBUG_OUTPUT */
       /* A multicast or broadcast packet was captured */
       P2pdPacketCaptured(ipPacket, nBytes);
 
@@ -791,7 +854,7 @@ DoP2pd(int skfd,
  * Data Used  : none
  * ------------------------------------------------------------------------- */
 int
-InitP2pd(struct interface *skipThisIntf)
+InitP2pd(struct interface_olsr *skipThisIntf)
 {
   if (P2pdUseHash) {
     // Initialize hash table for hash based duplicate IP packet check
@@ -862,6 +925,27 @@ SetP2pdUseHashFilter(const char *value,
   return 0;
 }
 
+/* -------------------------------------------------------------------------
+ * Function   : SetP2pdUseTtlDecrement
+ * Description: Set the TTL decrement lag for this plug-in
+ * Input      : value - parameter value to evaluate
+ *              data  - data associated with this parameter (unused in this app)
+ *              addon - additional parameter data
+ * Output     : none
+ * Return     : Always 0
+ * Data Used  : P2pdUseTtlDecrement
+ * ------------------------------------------------------------------------- */
+int
+SetP2pdUseTtlDecrement(const char *value,
+                     void *data __attribute__ ((unused)),
+                     set_plugin_parameter_addon addon __attribute__ ((unused)))
+{
+  assert(value != NULL);
+  P2pdUseTtlDecrement = atoi(value);
+
+  return 0;
+}
+
 /* -------------------------------------------------------------------------
  * Function   : AddUdpDestPort
  * Description: Set the UDP destination/port combination as an entry in the
@@ -913,7 +997,6 @@ AddUdpDestPort(const char *value,
       return -1;
     }
     break;
-  case AF_INET:
   default:
     res = inet_pton(AF_INET, destAddr, &addr4.sin_addr);
     if (!is_broadcast(addr4) && !is_multicast(addr4)) {
@@ -942,7 +1025,6 @@ AddUdpDestPort(const char *value,
            sizeof(addr6.sin6_addr.s6_addr));
     break;
   default:
-  case AF_INET:
     new->address.v4.s_addr = addr4.sin_addr.s_addr;
     break;
   }
@@ -1053,7 +1135,7 @@ dump_packet(unsigned char* packet, int length)
   }
   OLSR_PRINTF(1, "\n");
 }
-#endif
+#endif /* INCLUDE_DEBUG_OUTPUT */
 
 /* -------------------------------------------------------------------------
  * Function   : check_and_mark_recent_packet
