@@ -168,7 +168,7 @@ default_lq_ffeth_handle_lqchange(void) {
 }
 
 static void
-default_lq_parser_ffeth(struct olsr *olsr, struct interface *in_if, union olsr_ip_addr *from_addr)
+default_lq_parser_ffeth(struct olsr *olsr, struct interface_olsr *in_if, union olsr_ip_addr *from_addr)
 {
   const union olsr_ip_addr *main_addr;
   struct link_entry *lnk;
@@ -241,9 +241,17 @@ default_lq_ffeth_timer(void __attribute__ ((unused)) * context)
       // start with link-loss-factor
       ratio = fpmidiv(itofpm(link->loss_link_multiplier), LINK_LOSS_MULTIPLIER);
 
-      /* keep missed hello periods in mind (round up hello interval to seconds) */
+      /* don't forget missing hellos */
       if (tlq->missed_hellos > 1) {
-        received = received - received * tlq->missed_hellos * link->inter->hello_etime/1000 / LQ_FFETH_WINDOW;
+        uint32_t interval;
+
+        interval = tlq->missed_hellos * link->loss_helloint / 1000;
+        if (interval > LQ_FFETH_WINDOW) {
+          received = 0;
+        }
+        else {
+          received = (received * (LQ_FFETH_WINDOW - interval)) / LQ_FFETH_WINDOW;
+        }
       }
 
       // calculate received/total factor
