@@ -38,6 +38,7 @@
  *
  */
 
+#include <arpa/inet.h>
 #include <unistd.h>
 #include <signal.h>
 #include <sys/stat.h>
@@ -180,7 +181,7 @@ static int olsr_create_lock_file(bool noExitOnFail) {
 
   /* create file for lock */
   lock_fd = open(lock_file_name, O_WRONLY | O_CREAT, S_IRWXU);
-  if (lock_fd == 0) {
+  if (lock_fd < 0) {
     if (noExitOnFail) {
       return -1;
     }
@@ -316,7 +317,7 @@ int main(int argc, char *argv[]) {
   int i;
 
 #ifdef __linux__
-  struct interface *ifn;
+  struct interface_olsr *ifn;
 #endif /* __linux__ */
 
 #ifdef _WIN32
@@ -336,7 +337,7 @@ int main(int argc, char *argv[]) {
       olsrd_version, build_date, build_host);
 
   if (argc == 2) {
-    if (strcmp(argv[1], "-h") == 0 || strcmp(argv[1], "/?") == 0) {
+    if ((strcmp(argv[1], "-h") == 0) || (strcmp(argv[1], "/?") == 0)) {
       print_usage(false);
       exit(0);
     }
@@ -733,6 +734,9 @@ int main(int argc, char *argv[]) {
  *@param signo the signal that triggered this callback
  */
 void olsr_reconfigure(int signo __attribute__ ((unused))) {
+#ifndef _WIN32
+  int errNr = errno;
+#endif
   /* if we are started with -nofork, we do not want to go into the
    * background here. So we can simply stop on -HUP
    */
@@ -758,12 +762,15 @@ void olsr_reconfigure(int signo __attribute__ ((unused))) {
       olsr_syslog(OLSR_LOG_INFO, "RECONFIGURING!\n");
     }
   }
+#ifndef _WIN32
+  errno = errNr;
+#endif
   olsr_shutdown(0);
 }
 #endif /* _WIN32 */
 
 static void olsr_shutdown_messages(void) {
-  struct interface *ifn;
+  struct interface_olsr *ifn;
 
   /* send TC reset */
   for (ifn = ifnet; ifn; ifn = ifn->int_next) {
@@ -795,7 +802,10 @@ SignalHandler(unsigned long signo)
 static void olsr_shutdown(int signo __attribute__ ((unused)))
 #endif /* _WIN32 */
 {
-  struct interface *ifn;
+#ifndef _WIN32
+  int errNr = errno;
+#endif
+  struct interface_olsr *ifn;
   int exit_value;
 
   OLSR_PRINTF(1, "Received signal %d - shutting down\n", (int)signo);
@@ -917,6 +927,9 @@ static void olsr_shutdown(int signo __attribute__ ((unused)))
   exit_value = olsr_cnf->exit_value;
   olsrd_free_cnf(olsr_cnf);
 
+#ifndef _WIN32
+  errno = errNr;
+#endif
   exit(exit_value);
 }
 
@@ -1010,7 +1023,7 @@ static int olsr_process_arguments(int argc, char *argv[],
       NEXT_ARG;
       CHECK_ARGC;
 
-      if (inet_aton(*argv, &in) == 0) {
+      if (inet_pton(AF_INET, *argv, &in) == 0) {
         printf("Invalid broadcast address! %s\nSkipping it!\n", *argv);
         continue;
       }
