@@ -41,7 +41,7 @@
 
 #if defined __FreeBSD__ || defined __FreeBSD_kernel__ || defined __APPLE__ || defined __NetBSD__ || defined __OpenBSD__
 #define ifr_netmask ifr_addr
-#endif
+#endif /* defined __FreeBSD__ || defined __FreeBSD_kernel__ || defined __APPLE__ || defined __NetBSD__ || defined __OpenBSD__ */
 
 #include "ifnet.h"
 #include "ipcalc.h"
@@ -58,6 +58,7 @@
 #include "log.h"
 #include "link_set.h"
 
+#include <assert.h>
 #include <signal.h>
 #include <sys/types.h>
 #include <net/if.h>
@@ -73,7 +74,7 @@
  */
 #ifdef IPTOS_CLASS
 #undef IPTOS_CLASS
-#endif
+#endif /* IPTOS_CLASS */
 #define IPTOS_CLASS(class)    ((class) & IPTOS_CLASS_MASK)
 
 #define BUFSPACE  (127*1024)    /* max. input buffer size to request */
@@ -95,7 +96,7 @@ set_flag(char *ifname, short flag __attribute__ ((unused)))
 
   //printf("Setting flags for if \"%s\"\n", ifr.ifr_name);
 
-  if (!(ifr.ifr_flags & (IFF_UP | IFF_RUNNING))) {
+  if (!(ifr.ifr_flags & IFF_UP)) {
     /* Add UP */
     ifr.ifr_flags |= (IFF_UP | IFF_RUNNING);
     /* Set flags + UP */
@@ -115,7 +116,7 @@ check_interface_updates(void *foo __attribute__ ((unused)))
 
 #ifdef DEBUG
   OLSR_PRINTF(3, "Checking for updates in the interface set\n");
-#endif
+#endif /* DEBUG */
 
   for (tmp_if = olsr_cnf->interfaces; tmp_if != NULL; tmp_if = tmp_if->next) {
     if (tmp_if->host_emul)
@@ -128,7 +129,7 @@ check_interface_updates(void *foo __attribute__ ((unused)))
 #ifdef DEBUG
       /* Don't check this interface */
       OLSR_PRINTF(3, "Not checking interface %s\n", tmp_if->name);
-#endif
+#endif /* DEBUG */
       continue;
     }
 
@@ -148,11 +149,12 @@ check_interface_updates(void *foo __attribute__ ((unused)))
  * has been changed.
  *
  *@param iface the olsr_if struct describing the interface
+ *@return 1 when the interface changed, 0 if it did not change
  */
 int
 chk_if_changed(struct olsr_if *iface)
 {
-  struct interface *ifp;
+  struct interface_olsr *ifp;
   struct ifreq ifr;
   struct sockaddr_in6 tmp_saddr6;
   int if_changes;
@@ -160,7 +162,7 @@ chk_if_changed(struct olsr_if *iface)
 
 #ifdef DEBUG
   OLSR_PRINTF(3, "Checking if %s is set down or changed\n", iface->name);
-#endif
+#endif /* DEBUG */
 
   if (iface->host_emul)
     return -1;
@@ -188,7 +190,7 @@ chk_if_changed(struct olsr_if *iface)
    * First check if the interface is set DOWN
    */
 
-  if ((ifp->int_flags & IFF_UP) == 0 || (ifp->int_flags & IFF_RUNNING) == 0) {
+  if ((ifp->int_flags & IFF_UP) == 0) {
     OLSR_PRINTF(1, "\tInterface %s not up and running - removing it...\n", iface->name);
     goto remove_interface;
   }
@@ -255,21 +257,13 @@ chk_if_changed(struct olsr_if *iface)
 
 #ifdef DEBUG
     OLSR_PRINTF(3, "\tAddress: %s\n", ip6_to_string(&buf, &iface->cnf->ipv6_multicast.v6));
-#endif
+#endif /* DEBUG */
 
     if (memcmp(&tmp_saddr6.sin6_addr, &ifp->int6_addr.sin6_addr, olsr_cnf->ipsize) != 0) {
       OLSR_PRINTF(1, "New IP address for %s:\n", ifr.ifr_name);
       OLSR_PRINTF(1, "\tOld: %s\n", ip6_to_string(&buf, &ifp->int6_addr.sin6_addr));
       OLSR_PRINTF(1, "\tNew: %s\n", ip6_to_string(&buf, &tmp_saddr6.sin6_addr));
 
-      /* deactivated to prevent change of originator IP */
-#if 0
-      /* Check main addr */
-      if (memcmp(&olsr_cnf->main_addr, &tmp_saddr6.sin6_addr, olsr_cnf->ipsize) == 0) {
-        /* Update main addr */
-        memcpy(&olsr_cnf->main_addr, &tmp_saddr6.sin6_addr, olsr_cnf->ipsize);
-      }
-#endif
       /* Update address */
       memcpy(&ifp->int6_addr.sin6_addr, &tmp_saddr6.sin6_addr, olsr_cnf->ipsize);
       memcpy(&ifp->ip_addr, &tmp_saddr6.sin6_addr, olsr_cnf->ipsize);
@@ -293,7 +287,7 @@ chk_if_changed(struct olsr_if *iface)
     }
 #ifdef DEBUG
     OLSR_PRINTF(3, "\tAddress:%s\n", sockaddr4_to_string(&buf, &ifr.ifr_addr));
-#endif
+#endif /* DEBUG */
 
     if (memcmp
         (&((struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifp->int_addr))->sin_addr.s_addr, &((struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifr.ifr_addr))->sin_addr.s_addr,
@@ -304,14 +298,6 @@ chk_if_changed(struct olsr_if *iface)
       OLSR_PRINTF(1, "\tNew:%s\n", sockaddr4_to_string(&buf, &ifr.ifr_addr));
 
       ifp->int_addr = *(struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifr.ifr_addr);
-      /* deactivated to prevent change of originator IP */
-#if 0
-      if (memcmp(&olsr_cnf->main_addr, &ifp->ip_addr, olsr_cnf->ipsize) == 0) {
-        OLSR_PRINTF(1, "New main address: %s\n", sockaddr4_to_string(&buf, &ifr.ifr_addr));
-        olsr_syslog(OLSR_LOG_INFO, "New main address: %s\n", sockaddr4_to_string(&buf, &ifr.ifr_addr));
-        memcpy(&olsr_cnf->main_addr, &((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr, olsr_cnf->ipsize);
-      }
-#endif
       memcpy(&ifp->ip_addr, &((struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifr.ifr_addr))->sin_addr.s_addr, olsr_cnf->ipsize);
 
       /* we have to make sure that olsrd uses the new source address of this interface */
@@ -327,7 +313,7 @@ chk_if_changed(struct olsr_if *iface)
     }
 #ifdef DEBUG
     OLSR_PRINTF(3, "\tNetmask:%s\n", sockaddr4_to_string(&buf, &ifr.ifr_netmask));
-#endif
+#endif /* DEBUG */
 
     if (memcmp
         (&((struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifp->int_netmask))->sin_addr.s_addr, &((struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifr.ifr_netmask))->sin_addr.s_addr,
@@ -350,7 +336,7 @@ chk_if_changed(struct olsr_if *iface)
       }
 #ifdef DEBUG
       OLSR_PRINTF(3, "\tBroadcast address:%s\n", sockaddr4_to_string(&buf, &ifr.ifr_broadaddr));
-#endif
+#endif /* DEBUG */
 
       if (ifp->int_broadaddr.sin_addr.s_addr != ((struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifr.ifr_broadaddr))->sin_addr.s_addr) {
         /* New address */
@@ -382,7 +368,7 @@ remove_interface:
 int
 add_hemu_if(struct olsr_if *iface)
 {
-  struct interface *ifp;
+  struct interface_olsr *ifp;
   union olsr_ip_addr null_addr;
   uint32_t addr[4];
   struct ipaddr_str buf;
@@ -391,9 +377,9 @@ add_hemu_if(struct olsr_if *iface)
   if (!iface->host_emul)
     return -1;
 
-  ifp = olsr_malloc(sizeof(struct interface), "Interface update 2");
+  ifp = olsr_malloc(sizeof(struct interface_olsr), "Interface update 2");
 
-  memset(ifp, 0, sizeof(struct interface));
+  memset(ifp, 0, sizeof(struct interface_olsr));
 
   /* initialize backpointer */
   ifp->olsr_if = iface;
@@ -459,30 +445,14 @@ add_hemu_if(struct olsr_if *iface)
       olsr_syslog(OLSR_LOG_ERR, "Could not initialize socket... exiting!\n\n");
       olsr_cnf->exit_value = EXIT_FAILURE;
       kill(getpid(), SIGINT);
+
+      /* the kill command should not come back, just to be sure */
+      exit(1);
     }
 
   } else {
     /* IP version 6 */
     memcpy(&ifp->ip_addr, &iface->hemu_ip, olsr_cnf->ipsize);
-
-#if 0
-    /*
-     *We create one socket for each interface and bind
-     *the socket to it. This to ensure that we can control
-     *on what interface the message is transmitted
-     */
-
-    ifp->olsr_socket = gethcsocket6(&addrsock6, BUFSPACE, ifp->int_name);
-
-    join_mcast(ifp, ifp->olsr_socket);
-
-    if (ifp->olsr_socket < 0) {
-      fprintf(stderr, "Could not initialize socket... exiting!\n\n");
-      olsr_syslog(OLSR_LOG_ERR, "Could not initialize socket... exiting!\n\n");
-      olsr_cnf->exit_value = EXIT_FAILURE;
-      kill(getpid(), SIGINT);
-    }
-#endif
   }
 
   /* Send IP as first 4/16 bytes on socket */
@@ -550,26 +520,26 @@ if_basename(const char *name)
  * if it is set up and is of the correct type.
  *
  *@param iface the olsr_if struct describing the interface
- *@param so the socket to use for ioctls
- *
+ *@param debuglvl the debug level
+ *@return -1 when emulated (iface->host_emul), 0 on error, 1 on succes
  */
 int
 chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
 {
-  struct interface ifs, *ifp;
+  struct interface_olsr ifs, *ifp;
   struct ifreq ifr;
   union olsr_ip_addr null_addr;
   size_t name_size;
-#ifdef linux
+#ifdef __linux__
   int precedence = IPTOS_PREC(olsr_cnf->tos);
   int tos_bits = olsr_cnf->tos;
-#endif
+#endif /* __linux__ */
 
   if (iface->host_emul)
     return -1;
 
   memset(&ifr, 0, sizeof(struct ifreq));
-  memset(&ifs, 0, sizeof(struct interface));
+  memset(&ifs, 0, sizeof(struct interface_olsr));
   strscpy(ifr.ifr_name, iface->name, sizeof(ifr.ifr_name));
 
   OLSR_PRINTF(debuglvl, "Checking %s:\n", ifr.ifr_name);
@@ -582,7 +552,7 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
 
   ifs.int_flags = ifr.ifr_flags;
 
-  if ( ( (ifs.int_flags & IFF_UP) == 0) || ( (ifs.int_flags & IFF_RUNNING) == 0) ) {
+  if ( (ifs.int_flags & IFF_UP) == 0) {
     OLSR_PRINTF(debuglvl, "\tInterface not up & running - skipping it...\n");
     return 0;
   }
@@ -630,7 +600,7 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
 
 #ifdef __APPLE__
     ifs.int6_multaddr.sin6_scope_id = 0;
-#endif
+#endif /* __APPLE__ */
 
     OLSR_PRINTF(debuglvl, "\tMulticast: %s\n", ip6_to_string(&buf, &ifs.int6_multaddr.sin6_addr));
 
@@ -697,9 +667,6 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
 
   ifs.ttl_index = -32;          /* For the first 32 TC's, fish-eye is disabled */
 
-  /* Set up buffer */
-  net_add_buffer(&ifs);
-
   OLSR_PRINTF(1, "\tMTU - IPhdr: %d\n", ifs.int_mtu);
 
   olsr_syslog(OLSR_LOG_INFO, "Adding interface %s\n", iface->name);
@@ -778,13 +745,13 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
     join_mcast(&ifs, ifs.send_socket);
   }
 
-  ifp = olsr_malloc(sizeof(struct interface), "Interface update 2");
+  ifp = olsr_malloc(sizeof(struct interface_olsr), "Interface update 2");
 
   iface->configured = 1;
   iface->interf = ifp;
 
   /* XXX bad code */
-  memcpy(ifp, &ifs, sizeof(struct interface));
+  memcpy(ifp, &ifs, sizeof(struct interface_olsr));
 
   /* initialize backpointer */
   ifp->olsr_if = iface;
@@ -806,18 +773,28 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
   add_olsr_socket(ifp->olsr_socket, &olsr_input, NULL, NULL, SP_PR_READ);
   add_olsr_socket(ifp->send_socket, &olsr_input, NULL, NULL, SP_PR_READ);
 
-#ifdef linux
+#ifdef __linux__
   /* Set TOS */
 
   if (setsockopt(ifp->send_socket, SOL_SOCKET, SO_PRIORITY, (char *)&precedence, sizeof(precedence)) < 0) {
     perror("setsockopt(SO_PRIORITY)");
     olsr_syslog(OLSR_LOG_ERR, "OLSRD: setsockopt(SO_PRIORITY) error %m");
   }
-  if (setsockopt(ifp->send_socket, IPPROTO_IP, IP_TOS, (char *)&tos_bits, sizeof(tos_bits)) < 0) {
-    perror("setsockopt(IP_TOS)");
-    olsr_syslog(OLSR_LOG_ERR, "setsockopt(IP_TOS) error %m");
-  }
+  if (olsr_cnf->ip_version == AF_INET) {
+    if (setsockopt(ifp->send_socket, IPPROTO_IP, IP_TOS, (char *)&tos_bits, sizeof(tos_bits)) < 0) {
+      perror("setsockopt(IP_TOS)");
+      olsr_syslog(OLSR_LOG_ERR, "setsockopt(IP_TOS) error %m");
+    }
+  } else {
+    /* IP version 6 */
+#ifdef IPV6_TCLASS
+    if (setsockopt(ifp->send_socket, IPPROTO_IPV6, IPV6_TCLASS, (char *)&tos_bits, sizeof(tos_bits)) < 0) {
+      perror("setsockopt(IPV6_TCLASS)");
+      olsr_syslog(OLSR_LOG_ERR, "setsockopt(IPV6_TCLASS) error %m");
+    }
 #endif
+  }
+#endif /* __linux__ */
 
   /*
    *Initialize sequencenumber as a random 16bit value
@@ -864,6 +841,9 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
 
   ifp->mode = iface->cnf->mode;
 
+  /* Set up buffer */
+  net_add_buffer(ifp);
+
   /*
    *Call possible ifchange functions registered by plugins
    */
